@@ -17,8 +17,11 @@ struct srl_decoder {
     U32 flags;                          /* flag-like options: See SRL_F_DECODER_* defines in srl_decoder.c */
     UV max_recursion_depth;             /* Configurable limit on the number of recursive calls we're willing to make */
     UV max_num_hash_entries;            /* Configured maximum number of acceptable entries in a hash */
+    UV max_num_array_entries;           /* Configured maximum number of acceptable entries in an array */
+    UV max_string_length;               /* Configured maximum length of the string */
+    UV max_uncompressed_size;           /* Configured maximum size for uncompressed data */
     ptable_ptr ref_seenhash;            /* ptr table for avoiding circular refs */
-    ptable_ptr ref_thawhash;          /* ptr table for dealing with non ref thawed items */
+    ptable_ptr ref_thawhash;            /* ptr table for dealing with non ref thawed items */
     ptable_ptr ref_stashes;             /* ptr table for tracking stashes we will bless into - key: ofs, value: stash */
     ptable_ptr ref_bless_av;            /* ptr table for tracking which objects need to be bless - key: ofs, value: mortal AV (of refs)  */
     AV* weakref_av;
@@ -196,10 +199,19 @@ void srl_decoder_destructor_hook(pTHX_ void *p);
 #define SRL_DEC_OPT_STR_REFUSE_ZSTD                 "refuse_zstd"
 #define SRL_DEC_OPT_IDX_REFUSE_ZSTD                 13
 
+#define SRL_DEC_OPT_STR_MAX_NUM_ARRAY_ENTRIES       "max_num_array_entries"
+#define SRL_DEC_OPT_IDX_MAX_NUM_ARRAY_ENTRIES       14
+
+#define SRL_DEC_OPT_STR_MAX_STRING_LENGTH           "max_string_length"
+#define SRL_DEC_OPT_IDX_MAX_STRING_LENGTH           15
+
+#define SRL_DEC_OPT_STR_MAX_UNCOMPRESSED_SIZE       "max_uncompressed_size"
+#define SRL_DEC_OPT_IDX_MAX_UNCOMPRESSED_SIZE       16
+
 /* NOTE WELL: WHEN YOU ADD AN OPTION YOU **MUST** ADD A
  * CORRESPONDING CALL TO SRL_INIT_OPTION() to Decoder.xs */
 
-#define SRL_DEC_OPT_COUNT                           14
+#define SRL_DEC_OPT_COUNT                           17
 
 #if ((PERL_VERSION > 10) || (PERL_VERSION == 10 && PERL_SUBVERSION > 1 ))
 #   define MODERN_REGEXP
